from pathlib import Path
from typing import Any

def get_template_directories() -> set[Path]: ...
def reset_loaders() -> None: ...
def watch_for_template_changes(sender: object, **kwargs: Any) -> None: ...
def template_changed(sender: object | None, file_path: Path, **kwargs: Any) -> bool | None: ...
