from django.core.exceptions import ImproperlyConfigured as ImproperlyConfigured

class TaskException(Exception): ...
class InvalidTask(TaskException): ...
class InvalidTaskBackend(ImproperlyConfigured): ...
class TaskResultDoesNotExist(TaskException): ...
class TaskResultMismatch(TaskException): ...
