from typing import Any

from django.db import models
from typing_extensions import override

class AreaField(models.FloatField):
    geo_field: Any
    def __init__(self, geo_field: Any) -> None: ...
    @override
    def get_prep_value(self, value: Any) -> Any: ...
    @override
    def get_db_prep_value(self, value: Any, connection: Any, prepared: bool = ...) -> Any: ...
    def from_db_value(self, value: Any, expression: Any, connection: Any) -> Any: ...
    @override
    def get_internal_type(self) -> Any: ...

class DistanceField(models.FloatField):
    geo_field: Any
    def __init__(self, geo_field: Any) -> None: ...
    @override
    def get_prep_value(self, value: Any) -> Any: ...
    @override
    def get_db_prep_value(self, value: Any, connection: Any, prepared: bool = ...) -> Any: ...
    def from_db_value(self, value: Any, expression: Any, connection: Any) -> Any: ...
    @override
    def get_internal_type(self) -> Any: ...
